@@ -7,7 +7,7 @@ BEGIN {
   $Data::BitStream::XS::AUTHORITY = 'cpan:DANAJ';
 }
 BEGIN {
-  $Data::BitStream::XS::VERSION = '0.01';
+  $Data::BitStream::XS::VERSION = '0.02';
 }
 
 # parent is cleaner, and in the Perl 5.10.1 / 5.12.0 core, but not earlier.
@@ -342,10 +342,7 @@ sub code_is_universal {
   my $code = lc shift;
   my $param;  $param = $1 if $code =~ s/\((.+)\)$//;
   my $inforef = find_code($code);
-  if (!defined $inforef) {
-    warn "code_is_universal: unknown code '$code'\n";
-    return 0;
-  }
+  return unless defined $inforef;  # Unknown code.
   return $inforef->{'universal'};
 }
 
@@ -431,16 +428,19 @@ Bit streams are often used in data compression and in embedded products where
 memory is at a premium.
 
 This code provides a nearly drop-in XS replacement for the L<Data::BitStream>
-module.  If you do not need the ability to add custom codes, or the flexibility
-of the Moose/Mouse system, you can use this directly.
+module.  If you do not need the flexibility of the Moose/Mouse system, you can
+use this directly.
 
-The L<Data::BitStream> class will attempt to use this class if it is available.
-Most operations will be 50-100 times faster, while not sacrificing any of its
-flexibility, so it is highly recommended.
+As of version 0.03, the L<Data::BitStream> class will attempt to use this class
+if it is available.  Most operations will be 50-100 times faster, while not
+sacrificing any of its flexibility, so it is highly recommended.  In other
+words, if this module is installed, any code using L<Data::BitStream> will
+automatically speed up.
 
-There is not a lot of extra speed gained by using the XS class directly, so
-maximum portability and flexibility is had by using the L<Data::BitStream>
-class and installing this module to get the speed.
+While direct use of the XS class is a bit faster than going through Mouse/Moose,
+the vast majority of the benefit is internal.  Hence, for maximum portability
+and flexibility just install this module for the speed, and continue using the
+L<Data::BitStream> class as usual.
 
 =head1 METHODS
 
@@ -448,16 +448,87 @@ class and installing this module to get the speed.
 
 =over 4
 
+=item B< new >
+
+Creates a new object.  By default it has no associated file, is mode RW, and
+has maxlen 0 (no space allocated).  An optional hash of arguments may be
+supplied.  Examples:
+
+  $stream = Data::BitStream::XS->new( size => 10_000 );
+
+Indicates an initial start size of C<10,000> bits.  The normal behavior is to
+expand the data area as needed, but this can be used to make an initial
+allocation.  A small amount of time will be saved, and it will be more space
+efficient if the number of bits is known in advance.  This often will be a
+premature optimization.
+
+  $stream = Data::BitStream::XS->new( mode => 'ro' );
+
+The stream is opened as a read-only stream.  Attempts to open it for write will
+fail, hence all write / put methods will also fail.  This is most useful for
+opening a file for read, which will ensure no changes are made.
+
+Possible modes include C<'read' / 'r'>, C<'readonly' / 'ro'>, C<'write' / 'w'>,
+C<'writeonly' / 'wo'>, C<'append' / 'a'>, and C<'readwrite' / 'rw' / 'rdwr'>.
+
+  $stream = Data::BitStream::XS->new( file    => "c.bsc",
+                                      fheader => "HEADER $foo $bar",
+                                      mode    => 'w' );
+
+A file is associated with the stream.  Upon closing the file, going out of
+scope, or otherwise being destroyed, the stream will be written to the file,
+with the given header string written first.  While the current implementation
+writes at close time, later implementations may write as the stream is written
+to.
+
+  $stream = Data::BitStream::XS->new( file => "c.bsc",
+                                      fheaderlines => 1,
+                                      mode => 'ro' );
+
+A file is associated with the stream.  The contents of the file will be
+slurped into the stream.  The given number of header lines will be skipped
+at the start.  While the current implementation slurps the contents, later
+implementations may read from the file as the stream is read.
+
 =item B< maxbits >
 
 Returns the number of bits in a word, which is the largest allowed size of
 the C<bits> argument to C<read> and C<write>.  This will be either 32 or 64.
 
+It is theoretically possible that the maximum bits for this class and Perl
+do not match.  So this class may report 32-bit maxbits while Perl is 64-bit,
+and vice-versa.  This would usually happen only if it was loaded into a
+different Perl than it was compiled for.
+
+=item B< code_is_supported >
+
+Returns a hash of information about a code if it is known, and C<undef>
+otherwise.
+
+The argument is a text name, such as C<'Gamma'>, C<'Rice(2)'>, etc.
+
+=item B< code_is_universal >
+
+Returns C<undef> if the code is not known, C<0> if the code is non-universal,
+and a non-zero integer if it is universal.
+
+The argument is a text name, such as C<'Gamma'>, C<'Rice(2)'>, etc.
+
+A code is universal if there exists a constant C<C> such that C<C> plus the
+length of the code is less than the optimal code length, for all values.  What
+this typically means for us in practical terms is that non-universal codes
+are fine for small numbers, but their size increases rapidly, making them
+inappropriate when large values are possible (no matter how rare).  A
+classic non-universal code is Unary coding, which takes C<k+1> bits to
+store value C<k>.  This is very good if most values are 0 or near zero.  If
+we have rare values in the tens of thousands, it's not so great.  It is
+likely to be fatal if we ever come across a value of 2 billion.
+
 =back
 
 =head2 OBJECT METHODS (I<reading>)
 
-These methods are only value while the stream is in reading state.
+These methods are only valid while the stream is in reading state.
 
 =over 4
 
@@ -476,36 +547,44 @@ C<$bits> must be between C<1> and C<maxbits>.
 
 The position is advanced unless the second argument is the string 'readahead'.
 
+Attempting to read past the end of the stream is a fatal error.  However,
+readahead is allowed as it is speculative.  All positions past the end of
+the stream will always be filled with zero bits.
+
 =item B< skip($bits) >
 
 Advances the position C<$bits> bits.  Used in conjunction with C<readahead>.
 
+Attempting to skip past the end of the stream is a fatal error.
+
 =item B< read_string($bits) >
 
 Reads C<$bits> bits from the stream and returns them as a binary string, such
-as '0011011'.
+as C<'0011011'>.  Attempting to read past the end of the stream is a fatal
+error.
 
 =back
 
 =head2 OBJECT METHODS (I<writing>)
 
-These methods are only value while the stream is in writing state.
+These methods are only valid while the stream is in writing state.
 
 =over 4
 
 =item B< write($bits, $value) >
 
 Writes C<$value> to the stream using C<$bits> bits.  
-C<$bits> must be between C<1> and C<maxbits>.
+C<$bits> must be between C<1> and C<maxbits>, unless C<value> is 0 or 1, in
+which case C<bits> may be larger than C<maxbits>.
 
-The length is increased by C<$bits> bits.
+The stream length will be increased by C<$bits> bits.
 Regardless of the contents of C<$value>, exactly C<$bits> bits will be used.
 If C<$value> has more non-zero bits than C<$bits>, the lower bits are written.
-In other words, C<$value> will be masked before writing.
+In other words, C<$value> will be effectively masked before writing.
 
 =item B< put_string(@strings) >
 
-Takes one or more binary strings, such as '1001101', '001100', etc. and
+Takes one or more binary strings (e.g. C<'1001101'>, C<'001100'>) and
 writes them to the stream.  The number of bits used for each value is equal
 to the string length.
 
@@ -573,6 +652,17 @@ A read-only non-negative integer indicating the current length of the stream
 in bits.  It is advanced by C<write> and C<put> methods, as well as changed
 by C<from> and C<erase> methods.
 
+=item B< maxlen >
+
+A read-only non-negative integer indicating the current storage size of the
+stream in bits.  This will always be greater than or equal to the stream
+C<len>.  Applications will not normally need to know this.
+
+=item B< trim >
+
+Resizes the data to the stream C<len>, releasing all expansion space to
+the system.  Not normally needed.
+
 =item B< writing >
 
 A read-only boolean indicating whether the stream is open for writing or
@@ -624,13 +714,14 @@ If C<$count> is C<1> or not supplied, a single value will be read.
 If C<$count> is positive, that many values will be read.
 If C<$count> is negative, values are read until the end of the stream.
 
-C<get_> methods called in list context this return a list of all values read.
+C<get_> methods called in list context will return a list of all values read.
 Called in scalar context they return the last value read.
 
 C<put_> methods take one or more values as input after any optional
 parameters and write them to the stream.  All values must be non-negative
-integers that do not exceed the maximum encodable value (~0 for universal
-codes, parameter-specific for others).
+integers that do not exceed the maximum encodable value (typically ~0, but
+may be lower for some codes depending on parameter, and non-universal codes
+will be practically limited to smaller values).
 
 =over 4
 
@@ -722,13 +813,6 @@ the time efficient case where C<m = 2^k>.
 
 =item B< put_rice(sub { ... }, $k, @values) >
 
-Reads/writes one or more values from the stream in Rice coding, which is
-the time efficient case where C<m = 2^k>.
-
-=item B< get_rice(sub { ... }, $k [, $count]) >
-
-=item B< put_rice(sub { ... }, $k, @values) >
-
 Reads/writes one or more values from the stream in Rice coding using the
 supplied subroutine instead of unary coding, which can make them work with
 large outliers.  For example to use Omega coding for the base:
@@ -771,6 +855,24 @@ is an array reference which can be an anonymous array, for example:
   $stream->put_startstepstop( [3,2,9], @array );
   my @array3 = $stream->get_startstepstop( [3,2,9], -1);
 
+=item B< code_get($code, [, $count]) >
+
+=item B< code_put($code, @values ) >
+
+These methods wrap up all the previous encoding and decoding methods in an
+internal dispatch table.
+C<code> is a text name of the code, such as C<'Gamma'>, C<'Fibonacci'>, etc.
+Codes with parameters are called as C<'Rice(2)'>, C<'StartStop(0-0-2-4-14)'>,
+etc.
+
+  # $use_rice and $k obtained from options, parameters, or wherever.
+  my $code = $use_rice ? "Rice($k)" : "Delta";
+  my $nvalues = scalar @values;
+  $stream->code_put($code, @values);
+  # ....
+  my @vals = $stream->code_get($code, $nvalues);
+  print "Read $nvalues values with code '$code':  ", join(',', @vals), "\n";
+
 =back
 
 =head1 SEE ALSO
@@ -803,6 +905,12 @@ is an array reference which can be an anonymous array, for example:
 
 =item L<Data::BitStream::Code::StartStop>
 
+=item L<Data::BitStream::Code::Baer>
+
+=item L<Data::BitStream::Code::BoldiVigna>
+
+=item L<Data::BitStream::Code::ARice>
+
 =back
 
 =head1 AUTHORS
